# -*- coding: utf-8 -*-
# moebel.py

## Veraendernde Methoden greifen nur auf die
## eine Update-Methode zu.

from grafikfenster import *
from math import radians

from abc import ABC,abstractmethod # abstrakte Klasse realisieren

### ---------------------------------------------------
class Moebel(ABC):
    """Klasse Moebel
    Oberklasse für das Zeichnen und Bearbeiten von
    Möbel-Symbolen fuer den Raumplaner"""
    
    def __init__(self,
                 xPos,
                 yPos,
                 breite,
                 tiefe,
                 winkel,
                 farbe,
                 fuellfarbe,
                 sichtbar):
        """Konstruktor mit vordefinierten Parametern
        ist bei Moebel nicht sinnvoll"""
        self.__x=xPos
        self.__y=yPos
        self.__b=breite
        self.__t=tiefe
        self.__w=winkel
        self.__f=farbe
        self.__ff=fuellfarbe
        self.__s=sichtbar
        self.__ausgewaehlt=False
        self.__gruppe=None
        self.Update()
        
    @abstractmethod
    def GibFigur(self):
        """Definiert die zu zeichnende Figur."""
        return NotImplemented

    def GibZeichenPfad(self):
        return Zeichenflaeche.GibZeichenflaeche().GibGC().CreatePath()

    def Transformiere(self, path):
        """Transformiert den übergebenen Pfad"""
        gc = Zeichenflaeche.GibZeichenflaeche().GibGC()
        gc.PushState()
        gc.Translate(self.__x+self.__b/2, self.__y+self.__t/2)
        gc.Rotate(radians(self.__w))
        gc.Translate(-self.__b/2, -self.__t/2)
        transformation = gc.GetTransform()
        gc.PopState()
        path.Transform(transformation)
        if self.__gruppe!=None:
            return self.__gruppe.Transformiere(path)
        return path

    def SetzeGruppe(self, gruppe):
        self.__gruppe=gruppe

    def GibX(self):
        """Get-Methode fuer die xPosition"""
        return self.__x
    
    def GibY(self):
        """Get-Methode fuer die yPosition"""
        return self.__y
    
    def GibBreite(self):
        """Get-Methode fuer die Breite"""
        return self.__b
    
    def GibTiefe(self):
        """Get-Methode fuer die Tiefe"""
        return self.__t
    
    def GibOrientierung(self):
        """Get-Methode fuer die Orientierung"""
        return self.__w
    
    def GibFarbe(self, original=False):
        """Get-Methode fuer die Farbe, gibt bei True nicht grau"""
        if original: return self.__f
        if self.__ausgewaehlt: return "GREY"
        return self.__f
    
    def GibFuellFarbe(self, original=False):
        """Get-Methode fuer die Füllfarbe, gibt bei True nicht hell grau"""
        if original: return self.__ff
        if self.__ausgewaehlt: return "LIGHT GREY"
        return self.__ff
    
    def GibSichtbar(self):
        """Get-Methode fuer die Sichtbarkeit"""
        return self.__s

    def BewegeHorizontal(self, weite):
        """Veraendernde Methode fuer die x-Position"""
        self.__x += weite
        self.Update()
    
    def BewegeVertikal(self, weite):
        """Veraendernde Methode fuer die y-Position"""
        self.__y += weite
        self.Update()
    
    def Bewege(self, dx, dy):
        """Veraendernde Methode fuer die x-Position
        und die y-Position"""
        self.__x += dx
        self.__y += dy
        self.Update()
    
    def AendereBreite(self, neueBreite):
        """Veraendernde Methode fuer die Breite"""
        self.__b = neueBreite
        self.Update()
    
    def AendereTiefe(self, neueTiefe):
        """Veraendernde Methode fuer die Tiefe"""
        self.__t = neueTiefe
        self.Update()
    
    def Drehe(self, winkel):
        """Veraendernde Methode fuer die Orientierung [Winkel]"""
        self.__w += winkel
        self.Update()

    def AendereFarbe(self, neueFarbe):
        """Veraendernde Methode fuer die Farbe"""
        self.__f = neueFarbe
        self.Update()

    def AendereFuellFarbe(self, neueFarbe):
        """Verändernde Methode fuer die Füllfarbe"""
        self.__ff = neueFarbe
        self.Update()

    def Verberge(self):
        """Veraendernde Methode fuer die Sichtbarkeit mit Wert False"""
        self.__s = False
        self.Update()

    def Zeige(self):
        """Veraendernde Methode fuer die Sichtbarkeit mit Wert True"""
        self.__s = True
        self.Update()

    def Waehle(self, auswaehlen):
        if auswaehlen: self.__ausgewaehlt=True
        else: self.__ausgewaehlt=False
        self.Update()

    def Update(self):
        """aktualisiert die Darstellung."""
        if self.GibSichtbar():
            Zeichenflaeche.GibZeichenflaeche().Zeichne(self)
        else:
            Zeichenflaeche.GibZeichenflaeche().Entferne(self)
        
### ---------------------------------------------------
class MoebelApp(wx.App):
    """Test-Anwendung speziell fuer Stuhl"""
    def OnInit(self):
        self.fenster = GrafikFenster(None, "Raumplaner-Grafik")
        self.SetTopWindow(self.fenster)
        self.fenster.Show(True)
        self.fenster.panel.Refresh()
        self.fenster.ZeigeShellFrame()
        #self.fenster.ZeigeFillingFrame()
        self.TestAnwendung()
        return True

    def TestAnwendung(self):
        """Allein fuer die Testanwendung:
        sollte nicht gehen!"""
        global moebel
        try:
            moebel=Moebel(70, 30, 120, 60, 0, 'red', True)
        except Exception as e:
            print( str(e) )
        
### ---------------------------------------------------
if __name__ == '__main__':
    app = MoebelApp(redirect=False)
    # Parameterwert True, wenn Ausgaben in der Standard E/A angezeigt werden sollen
    app.MainLoop()
